//------------------------------------------------------------
//---        HILAND L/C meter firmware for AVR             ---
//---          ATmega328P / ATmega8A version               ---
//---                   2019-11-25                         ---
//------------------------------------------------------------
//---

//===================================================================
// NOTES:
// NOW IT WORK WITH ATMEGA 328P AND ATMEGA 8A
//
//---
// TO DO:
// ... ALL DONE !


//
//===================================================================
//

#define m328p

#ifdef m328p
.include "m328P-def.inc"
#else
.include "m8A-def.inc"
#endif

//------------------------
//---  for ATMega 8A  ----
//------------------------
//        LO   HI   LOCK -
// fuses: 0x7D 0xDF 0x3F -
//------------------------

//------------------------------
//---    for ATMega 328P    ----
//------------------------------
//        LO   HI   EXT   LOCK -
// fuses: 0xFD 0xDF 0xFC  0x3F -
//------------------------------

//---------------------
// Lcd = PORTC.3,2,1,0
// RS=PORTC.5
// EN=PORTC.4
// LCD_TYPE = 16 * 2
//---------------------

// !!! "SYS_FREQ" MUST BE CHANGED TO PHYSICAL CRYSTAL !!!
.EQU	SYS_FREQ	= 8 		; czstotliwo pracy w MHz
//.EQU	SYS_FREQ	= 20		; czstotliwo pracy w MHz


.EQU K_LCD		= DDRB	; rejestr kierunku portu wywietlacza
.EQU O_LCD		= PORTB	; rejestr wyjciowy portu wywietlacza
.EQU LCD_RS		= 0		; nr linii dla sygnau RS
.EQU LCD_EN		= 1		; nr linii dla sygnau EN
.EQU LCD_DB4	= 2		; nr linii dla sygnau DB4
.EQU LCD_DB5	= 3		; nr linii dla sygnau DB5
.EQU LCD_DB6	= 4		; nr linii dla sygnau DB6
.EQU LCD_DB7	= 5		; nr linii dla sygnau DB7



//--------------------------
//--- our private deines ---
//--------------------------
#ifdef m328p
.equ     RAMSTART   = 0x0100
#else
.equ     RAMSTART   = 0x60
#endif
//-


//---------------------------------------------
//--- controll for change microcontroller's ---
//---------------------------------------------



//----------------------------------------------------------
.equ     ADMUX_START_VALUE    = 0b01000000  // start value for ADMUX
.equ     ADCSR_START_VALUE    = 0b10001111  // start value for ADCSR
.equ     SMCR_START_VALUE     = 0b00000011  // start value for ADCSR

.equ     DDRC_START_VALUE     = 0b00000000
.equ     PORTC_START_VALUE    = 0b00000001


.equ     ACSR_START_VALUE    = (1<<ACD)  // start value for ACSR

.equ     DIRECTION_TC1_PORT_START_VALUE    = 0b01011000  // initial inout/output value of PORTD
.equ     DIRECT_TC1_PORT = DDRD


.equ     TC1_PORT_START_VALUE   = 0b10111111  // initial inout/output value of PORTD
.equ     TC1_PORT = PORTD

#ifdef m328p
.equ     TIMSK1_START_VALUE  = (1<<TOIE1)  // IRQ TC1 OVERFLOW enable value 
.EQU     TIMSK0_START_VALUE  = (1<<TOIE0)  // IRQ TC0 OVERFLOW enable value 
#else
.EQU     TIMSK_START_VALUE  = (1<<TOIE0) | (1<<TOIE1)
#endif

.equ     TIMER1_START_VALUE  = (1<<CS12) | (1<<CS11) | (1<<CS10) // COUNT EXTERNAL TICKS ON RISING EDGE
.EQU     TIMER0_START_VALUE  = (1<<CS00)  //NO PRESCALE
//---------------------------------------------------------

.EQU    EE_C_OWN = 0x10
.EQU    EE_L_OWN = 0x12
.EQU    EE_C_REF = 0x14
.EQU    EE_F1    = 0x16
.EQU    EE_F2    = 0x1A
.EQU    EE_R1    = 0x1E
.EQU    EE_R2    = 0x20

//---------------------------------------------------------
.def     mod10         =     r16  // helps variable for divide | multiply
.def     cntr_1        =     r17  // helps variable for divide | multiply
.def     divisor       =     r18  // disor for divide proc
//--------

//--------
.EQU    RX_ADD = 0x01
//------
.def	RX0	 =	r1
.def	RX1	 =	r2
.def	RX2	 =	r3
.def	RX3	 =	r4
.def	RX4	 =	r5
.def	RX5	 =	r6
.def	RX6	 =	r7
.def	RX7	 =	r8

.EQU RY_ADD = 0x09
//-------
.def	RY0	 =	r9
.def	RY1	 =	r10
.def	RY2	 =	r11
.def	RY3	 =	r12
.def	RY4	 =	r13
.def	RY5	 =	r14

.def	RM0	 =	r15
.def	RM1	 =	r16
.def	RM2	 =	r17
.def	RM3	 =	r18
.def	RM4	 =	r19
.def	RM5	 =	r20
//--------
        

//-------------------------------------------------
//=================================================
		.DSEG
.ORG     RAMSTART
//--------
VAR_START:			.BYTE 1     // UNUSED
//--------

OUT_ASCII_BUFF:		.BYTE 10    // 10 bytes for ASCII decimal string 32bit unsigned value
LEN_ASCII_BUFF:		.BYTE 1     // length of ASCII digits
MY_LONG_INT:		.BYTE 4     // 4 bytes LONG INTEGER - 32bit unsigned value 
MY_JOB:				.BYTE 1     // 0 = Cx MEASSURE | 1 = Lx MEASSURE | 2 = CE MEASSURE
RING_LONG:			.BYTE 1     // INDICATOR OF BUZZER

FREQ1:				.BYTE 4     // FREQ OF L/C CIRCUIT
FREQ2:				.BYTE 4     // FREQ OF CALIBRATION
FREQ3:				.BYTE 4     // FREQ OF L MEASSURE
C_REF:				.BYTE 2     // REF CAP VALUE
C_OWN:				.BYTE 2     // OWN CAP VALUE
L_OWN:				.BYTE 2     // OWN CAP VALUE
REZ1:				.BYTE 2     // 2000 OHM
REZ2:				.BYTE 2     // 200 OHM
REZ3:				.BYTE 2     // R3 =100 * R2 = 20000
FREQ1X:				.BYTE 4
SIGN_F:				.BYTE 1
FR3_HI:				.BYTE 1
CALC_PLACE:			.BYTE 4     // BCD DATA TO CALC AND CONVERT
POINT_PLACE:		.BYTE 1
BIG_CAP:			.BYTE 4
//--------
VAR_END:			.BYTE 1     // UNUSED

//--------
//////////////////////////////////////////////////////////////
//				cp	  R16, R20
//				cpc	  R17, R21
//				cpc	  R18, R22
//				CPC   R19, R23
//
//				R16-19 < R20-23 C=1
//				R16-19 = R20-23 C=0 | Z=1
//				R16-19 > R20-23 C=0 | Z=0
//////////////////////////////////////////////////////////////


          .CSEG
.ORG     0x0000

//------------------------------
//-- vectors of interrupts -----
//------------------------------
#ifdef m328p
				jmp     _RESET_PROC      // 0x0000 jmp     _RESET_PROC
				jmp     _EXT_INT0        // 0x0002 jmp INT0 ; IRQ0
				jmp     _RESET_PROC      // 0x0004 jmp INT1 ; IRQ1
				jmp     _RESET_PROC      // 0x0006 jmp PCINT0 ; PCINT0
				jmp     _RESET_PROC      // 0x0008 jmp PCINT1 ; PCINT1
				jmp     _RESET_PROC      // 0x000A jmp PCINT2 ; PCINT2
				jmp     _RESET_PROC      // 0x000C jmp WDT ; Watchdog Timeout
				jmp     _RESET_PROC      // 0x000E jmp TIM2_COMPA ; Timer2 CompareA
				jmp     _RESET_PROC      // 0x0010 jmp TIM2_COMPB ; Timer2 CompareB
				jmp     _RESET_PROC      // 0x0012 jmp TIM2_OVF ; Timer2 Overflow
				jmp     _RESET_PROC      // 0x0014 jmp TIM1_CAPT ; Timer1 Capture
				jmp     _RESET_PROC      // 0x0016 jmp TIM1_COMPA ; Timer1 CompareA
				jmp     _RESET_PROC      // 0x0018 jmp TIM1_COMPB ; Timer1 CompareB
				jmp     TIM1_OVF         // 0x001A jmp TIM1_OVF ; Timer1 Overflow
				jmp     _RESET_PROC      // 0x001C jmp TIM0_COMPA ; Timer0 CompareA
				jmp     _RESET_PROC      // 0x001E jmp TIM0_COMPB ; Timer0 CompareB
				jmp     TIM0_OVF         // 0x0020 jmp TIM0_OVF ; Timer0 Overflow
				jmp     _RESET_PROC      // 0x0022 jmp SPI_STC ; SPI Transfer Complete
				jmp     _RESET_PROC      // 0x0024 jmp USART_RXC ; USART RX Complete
				jmp     _RESET_PROC      // 0x0026 jmp USART_UDRE ; USART UDR Empty
				jmp     _RESET_PROC      // 0x0028 jmp USART_TXC ; USART TX Complete
				RETI                     // 0x002A jmp ADC ; ADC Conversion Complete
				jmp     _RESET_PROC      // 0x002C jmp EE_RDY ; EEPROM Ready
				jmp     _RESET_PROC      // 0x002E jmp ANA_COMP ; Analog Comparator
				jmp     _RESET_PROC      // 0x0030 jmp TWI ; 2-wire Serial
				jmp     _RESET_PROC      // 0x0032 jmp SPM_RDY ; SPM Ready;
#else
				rjmp     _RESET_PROC     // $000 rjmp RESET ; Reset Handler
				rjmp     _EXT_INT0       // $001 rjmp EXT_INT0 ; IRQ0 Handler
				rjmp     _RESET_PROC     // $002 rjmp EXT_INT1 ; IRQ1 Handler
				rjmp     _RESET_PROC     // $003 rjmp TIM2_COMP ; Timer2 Compare Handler
				rjmp     _RESET_PROC     // $004 rjmp TIM2_OVF ; Timer2 Overflow Handler
				rjmp     _RESET_PROC     // $005 rjmp TIM1_CAPT ; Timer1 Capture Handler
				rjmp     _RESET_PROC     // $006 rjmp TIM1_COMPA ; Timer1 CompareA Handler
				rjmp     _RESET_PROC     // $007 rjmp TIM1_COMPB ; Timer1 CompareB Handler
				rjmp     TIM1_OVF        // $008 rjmp TIM1_OVF ; Timer1 Overflow Handler
				rjmp     TIM0_OVF        // $009 rjmp TIM0_OVF ; Timer0 Overflow Handler
				rjmp     _RESET_PROC     // $00a SPI Transfer Complete Handler
				rjmp     _RESET_PROC     // $00b USART RX Complete Handler
				rjmp     _RESET_PROC     // $00c UDR Empty Handler
				rjmp     _RESET_PROC     // $00d USART TX Complete Handler
				RETI                     // $00e ADC Conversion Complete Handler
				rjmp     _RESET_PROC     // $00f EEPROM Ready Handler
				rjmp     _RESET_PROC     // $010 Analog Comparator Handler
				rjmp     _RESET_PROC     // $011 Two-wire Serial Interface Handler
				rjmp     _RESET_PROC     // $012 Store Program Memory Ready Handler
#endif
//==============================================================

//=======================================
_RESET_PROC:

				cli      // DISABLE IRQ

				ldi      yh,high(RAMEND)
				out      SPH,yh  // stack init
				ldi      yl,low(RAMEND)
				out      SPL,yl  // stack init
				dec      yh
//====================================================

				rcall    stop_meassure  //

				LDI      R20, DDRC_START_VALUE
				OUT      DDRC, R20
				LDI      R20, PORTC_START_VALUE
				OUT      PORTC, R20

				ldi      ZL, DIRECTION_TC1_PORT_START_VALUE
				out      DIRECT_TC1_PORT,ZL
				ldi      ZL, TC1_PORT_START_VALUE
				out      TC1_PORT,ZL


				RCALL    CLEAR_JOB
				RCALL    WAIT50MS
				rcall    LcdIni

				ldi      zl, low(welcome * 2)
				ldi      zh, high(welcome * 2) 
				rcall    WriteLCDString
				RCALL    _from_line2
				ldi      zl, low(welcome2 * 2)
				ldi      zh, high(welcome2 * 2) 
				rcall    WriteLCDString
				ldi      zl, low(RAMSTART+1)
				ldi      zh, high(RAMSTART+1)

//---
				ldi      r16, LOW(VAR_END - VAR_START)   // clear data ram 32 bytes of variables, string etc...
				clr      XH
				st       Z+, XH
				dec      r16
				brne     PC-2
				RCALL    COPY_VARIABLES
//===================================================

//===================================================
				RCALL    R3_CALC           // R3 =100 * R2 = 20000

				RCALL    DO_MEASS          // 1 SEC DELAY

				RCALL    RUN_CALIBRATE     // DO CALIBRATION
COMPENSATE:
				rcall    LcdIni
				ldi      zl, low(welcome3 * 2)
				ldi      zh, high(welcome3 * 2) 
				rcall    WriteLCDString
				RCALL    _from_line2
				RCALL    PRINT_L_OWN
				RCALL    PRINT_C_OWN
//=====================================
//==========   repeat main loop =======
//=====================================
starter:

				LDS      R16, MY_JOB
				CPI      R16,2
				BRCS     PC+2
				RJMP     CE_MEASSURE
				CPI      R16, 0
				BRNE     NOT_CX
				SBI      PORTD, 3 // DISABLE CCALL AND CHARGE BY 2K OHM
				SBI      PORTD, 4 // ENABLE Cx MEAS
				RJMP     SETUP_DONE
NOT_CX:         SBI      PORTD, 3 // DISABLE CCALL AND CHARGE BY 2K OHM
                CBI      PORTD, 4 // ENABLE Lx MEAS
SETUP_DONE:
				rcall    start_meassure  // OK - let's go rocket :)
//====================================
selfee:
				SBIS     PIND,7     // SW1 PRESSED?
				RJMP     CHANGE_JOB
				TST      R11
				BREQ     selfee
				CLI
				RCALL    COPY_REG_TO_FREQ3

				rcall    LcdIni
				RCALL    print_f

				LDS      R16, MY_JOB
				CPI      R16, 1
				BREQ     LX_MEASS

				SBI      PORTD, 4 // K1 TO Cx
				RCALL    CX_CALC
				rcall    PRINT_PF
				RJMP     starter
				
LX_MEASS:
				CBI      PORTD, 4  // K1 TO Lx
				RCALL    LX_CALC
				RCALL    CHECK_FREQ
				BREQ     starter
				rcall    PRINT_LX
/////////////////////////////////////////////////////////////////////
				rjmp     starter   // OK repeat over and over again
/////////////////////////////////////////////////////////////////////

//==================================================================
//------------------------
//------ IRQ SERVICE -----
//------------------------
_EXT_INT0:

				PUSH    R12
				IN      R12, SREG
#ifdef m328p
				LDS     R1, TCNT1L
				LDS     R2, TCNT1H
#else
				IN      R1, TCNT1L
				IN      R2, TCNT1H
#endif
				RCALL   stop_meassure
				INC     R10

				OUT     SREG, R12
				POP     R12
				RETI
//------------------------
//------ IRQ SERVICE -----
//------------------------
TIM1_OVF:
				PUSH    R20
				IN      R20, SREG

				INC     R3
				BRNE    TIM1_EX
				INC     R4
TIM1_EX:

				OUT SREG, R20
				POP R20
				reti
//------------------------
//------ IRQ SERVICE -----
//------------------------
TIM0_OVF:
//-----------------------------------------------
//-----      read captured TIMER1 value     -----
//-----      and store her to variable      -----
//-----------------------------------------------
// !!! VALUE MUST BE CHANGED TO PHYSICAL CRYSTAL !!!
// SYS_FREQ DIV BY 256
// 20 000 000 DIV BY 256 = 0x01312D
//  8 000 000 DIV BY 256 = 0x7A12


				PUSH    R12
				IN      R12, SREG
#ifdef m328p
				LDS     R1, TCNT1L
				LDS     R2, TCNT1H
#else
				IN      R1, TCNT1L
				IN      R2, TCNT1H
#endif
				INC     R9
				BRNE    T0_OV1
				INC     R10
;---

T0_OV1:			PUSH    R20
				MOV     R20, R10
				CPI     R20,0x7A
				BRNE    IRQ_NO

				MOV     R20, R9
				CPI     R20,0x12
				BRNE    IRQ_NO

				RCALL   stop_meassure
				INC     R11

IRQ_NO:			POP     R20
				OUT     SREG, R12
				POP     R12
				RETI
//-----------------------------------

//----------------------------
//------ IRQ SERVICE END -----
//----------------------------
//=================================================================
//---
PRINT_LX:
				LDI      R24, 'L'
				RCALL    _Lch

				RCALL    SIGN_PRT
								
				RCALL    get_string_pointers
				CPI      R17, 1
				BRNE     TEST_2_DIGITS
				RCALL    print_0_dot
				RCALL    print_0
				RCALL    print_last_digit
				RJMP     UH_PRINT
//---
TEST_2_DIGITS:  CPI      R17, 2
				BRNE     TEST_mH
				RCALL    print_0_dot
				RCALL    print_two_last_digit
				RJMP     UH_PRINT
TEST_mH:
				CPI      R17,6
				BRCS     PRINT_UH
				SUBI     R17, 5
				CLR      R18
				RCALL    print_from_buff
				RCALL    print_dot
				LDI      R17,3
				CLR      R18
				RCALL    print_from_buff
				LDI      R24, 'm'
				RJMP     MH_PRINT
PRINT_UH:
				RCALL    print_meassure_2digits
UH_PRINT:
				LDI      R24, 'u'
MH_PRINT:
				RCALL    _Lch
				LDI      R24, 'H'
				RJMP    _Lch
//---



//---
PRINT_NF:
				LDI      R18,4
				RCALL    PRINT_R18_LONG
				rcall    print_dot
				LDI      R18,1
				RCALL    PRINT_R18_LONG	
BIK:
				LDI      R24, 'n'
				RJMP     FARADS_PRINT
PRINT_R18_LONG:

				ld       r24, Z+
				rcall    _Lch
				dec      r17
				cp       r17,r18
				brne     PC-4
				RET
SIGN_PRT:
				LDI      R24, 'x'
				RCALL    _Lch
				LDI      R24, '='
				RCALL    _Lch
				LDS      R24, SIGN_F
				TST      R24
				BREQ     PC+3
				LDI      R24, '-'
				RCALL    _Lch
				RET
//---
PRINT_PF:
				LDI      R24, 'C'
				RCALL    _Lch


				RCALL    SIGN_PRT

				RCALL    get_string_pointers
				CPI      R17,1
				BRNE     MORE_DIGITS
				RCALL    print_0_dot
				RCALL    print_last_digit
				RJMP     LP01
MORE_DIGITS:
				CPI      R17,6
				BRCC     PRINT_NF
				DEC      R17
				CLR      R18
				RCALL    print_from_buff
				RCALL    print_dot
				RCALL    print_last_digit
LP01:
				LDI      R24, 'p'
FARADS_PRINT:
				PUSH     R24
				RCALL    ONE_SPACE
				POP      R24
				RCALL    _Lch
				LDI      R24, 'F'
				RJMP    _Lch
       
//---
PRINT_C_OWN:
				RCALL    ONE_SPACE
				LDS      R1, C_OWN
				LDS      R2, C_OWN+1
MAKE_PRT:
				RCALL    CONVERT_16BIT
				RCALL    print_meassure_0digits
				RJMP     LP01
//---
PRINT_L_OWN:
				LDS      R1, L_OWN
				LDS      R2, L_OWN+1
				RCALL    CONVERT_16BIT
				RJMP     PRINT_UH
//---
CONVERT_16BIT:
				rcall    CLEAR_MY_LONG_INT
				
				STS      MY_LONG_INT, R1
				STS      MY_LONG_INT+1, R2

				RJMP    get_string_pointers 
//-----------------------------------------
print_f:
				RCALL    FREQ3_TO_LONG
				LDS      R0, SIGN_F
				TST      R0
				BREQ     PC + 2
				RCALL    FREQ1X_TO_LONG
				rcall    get_string_pointers 

				rcall    _from_line2

				LDI      R24, 'F'
				RCALL    _Lch
				LDI      R24, '='
				RCALL    _Lch
PRINT_F_HZ:
				RCALL    print_meassure_0digits

				RCALL    ONE_SPACE
				LDI      R24, 'H'
				RCALL    _Lch
				LDI      R24, 'z'
				RCALL    _Lch
				RJMP     _from_line1
//---
get_string_pointers:
				rcall    BIN32u_2_ASCII   // first make the properly string

				lds      r17, LEN_ASCII_BUFF
				ldi      zl,low(OUT_ASCII_BUFF+10)
				ldi      zh,high(OUT_ASCII_BUFF+10)
				sub      zl, r17
				clr      r0
				sbc      zh, r0
				ret
//=====================================
//---
print_three_last_digit:
				cpi      r18, 2
				breq     print_two_last_digit

				lds      r24, (OUT_ASCII_BUFF+7)
				rcall    _Lch
print_two_last_digit:
				lds      r24, (OUT_ASCII_BUFF+8)
				rcall    _Lch
print_last_digit:
				lds      r24, (OUT_ASCII_BUFF+9)
				rcall    _Lch
				RET
//---
print_0_dot:
				rcall    print_0
print_dot:     
				ldi      r24, '.'
				rjmp     _Lch
//-
print_0:
				ldi      r24, 0x30
				rjmp     _Lch
//-------------------------


//-----------------------------
//- R18 MANY DIGITS AFTER DOT -
//- R19 LENGTH                -
//-----------------------------
print_meassure_0digits:
				ldi      r18, 0 // MANY DIGITS AFTER DOT
				MOV      R19, R17
				rjmp     print_fixed_point
print_meassure_1digits:
	
				ldi      r18, 1 // MANY DIGITS AFTER DOT
				LDI      r19, 4
				rjmp     print_fixed_point
print_meassure_2digits:
				ldi      r18, 2 // MANY DIGITS AFTER DOT
				ldi      r19, 3
				rjmp     print_fixed_point
print_meassure_3digits:
				ldi      r18, 3 // MANY DIGITS AFTER DOT
				ldi      r19, 4
print_fixed_point:
				cp       r17, r19 //r17=LEN_ASCII_BUFF
				brcc     print_from_buff
				rcall    print_0_dot
				cp       r17, r18
				breq     print_three_last_digit
				rcall    print_0
				cpi      r17, 2
				breq     print_two_last_digit
				rcall    print_0
				rjmp     print_last_digit
//---
print_from_buff:
				ld       r24, Z+
				rcall    _Lch
				dec      r17
				cp       r17,r18
				brne     print_from_buff
				TST      R18
				BRNE     PC+2
				RET
				rcall    print_dot
				rjmp     print_three_last_digit
//-------------------------

//================================================
stop_meassure:
//--------------------------
//--- disable interrupts ---
//---    stop   TIMER1   ---
//--------------------------

				PUSH     ZL
				clr      zl
#ifdef m328p
				sts      TIMSK1, zl
				sts      TCCR1B, zl
				sts      TCCR1A, zl
				STS      TIMSK0, ZL
				OUT      TCCR0B, zl
				OUT      TCCR0A, zl
				STS      EICRA, ZL
				OUT      EIMSK, ZL
				sts      TCNT1H, ZL
				sts      TCNT1L, ZL
				OUT      TCNT0, ZL
#else
				out      TIMSK, zl
				out      TCCR1B, zl
				out      TCCR1A, zl
				OUT      TCCR0, zl
				OUT      MCUCR, ZL
				OUT      GICR, ZL
				out      TCNT1H, ZL
				out      TCNT1L, ZL
				OUT      TCCR0, ZL
				OUT      TCNT0, ZL
#endif
				ldi      zl, ACSR_START_VALUE
				out      ACSR, zl
				RCALL    CLEAR_IRQ_FLAGS

				POP      ZL
				ret
//------------------------------------------------
start_meassure:
//------------------------
//--- block interrupts ---
//------------------------
				cli
				rcall    stop_meassure
				RCALL    FREQ1_TO_FREQ1X
				rcall    CLEAR_MY_LONG_INT
				RCALL    copy_long_to_registers
				CLR      R9
				CLR      R10
				CLR      R11
//---------------------------------
//-----        Timer1 = 0    ------
//---------------------------------
#ifdef m328p
//--------------------------------------------------------
				ldi      ZL, TIMSK1_START_VALUE
				sts      TIMSK1, ZL
				ldi      ZL, TIMSK0_START_VALUE
				sts      TIMSK0, ZL

				LDI      R24, TIMER0_START_VALUE
				ldi      ZL,TIMER1_START_VALUE      
				sts      TCCR1B, ZL                 // START TIMER1
				OUT      TCCR0B, R24
#else
				ldi      ZL, TIMSK_START_VALUE
				out      TIMSK,  ZL

				LDI      R24, TIMER0_START_VALUE
				ldi      ZL,TIMER1_START_VALUE      
				out      TCCR1B, ZL                 // START TIMER1
				OUT      TCCR0, R24
#endif
//-----------------------
//--- clear IRQ flags ---
//-----------------------
				RCALL    CLEAR_IRQ_FLAGS
//---------
//-------------------------
//--- unlock interrupts ---
//-------------------------
				sei  // ok - wait for TC1
				ret
//***********************************************
CLEAR_IRQ_FLAGS:
				
				PUSH     R24
#ifdef m328p
				in       R24,TIFR1
				out      TIFR1, R24
				in       R24,TIFR0
				out      TIFR0, R24
				in       R24,EIFR
				out      EIFR, R24
#else
				in       R24,TIFR
				out      TIFR, R24
				in       R24,GIFR
				out      GIFR, R24
#endif
				POP      R24
				RET
//-----------------------------------------------------------------------------:
// 32bit Unsigned convert BIN to ASCII
//
// Register Variables
//  Call:  
//         var[1:4]           = 32bit unsigned value (0x00000000..0xffffffff)
//         mod10              = <don't care> (high register must be allocated)
//         cntr_1             = <don't care> (high register must be allocated)
//
//
//  Result:
//         OUT_ASCII_BUFF[0:9] = 10 bytes will be overwrite as ASCII decimal bin representation
//         var[1:4]            = (0x00000000)
//         mod10               = length of ASCII numbers
//         cntr_1              = 0
//         xl,xh               = @OUT_ASCII_BUFF first ASCII number
//
//---------------
BIN32u_2_ASCII:
				rcall   copy_long_to_registers

				ldi     xl, low(OUT_ASCII_BUFF+10)
				ldi     xh, high(OUT_ASCII_BUFF+10)

				ldi     divisor, 10   //divide by 10

BIN_TO_ASCII:
				rcall   div32u_by_8u  //make divide by 10
//---------------------------
				ori     mod10, 0x30   // O_o HERE we gotta our the rest of div10 :] o_O
				st      -x,mod10      // store as ASCII NUMBER
				cpi     xl, low(OUT_ASCII_BUFF)
				brne    BIN_TO_ASCII

				ldi     mod10,9

clr_lead:      
				ld      cntr_1, X
				cpi     cntr_1, 0x30
				brne    stop_it
				ldi     cntr_1, 0x20
				st      x+, cntr_1
				dec     mod10
				brne    clr_lead
stop_it:
				inc     mod10
				sts     LEN_ASCII_BUFF, mod10  // save how many bytes ASCII NUMBERS strig is here
				ret
//---
//------------------------------------------------------------------
// ---------- like above - prepare our fixed point value  ----------
//------------------------------------------------------------------

//-------------
//------------------------------------------------------------
//--- divide unsigned 32bit value by 8bit unsigned divisor ---
//------------------------------------------------------------
//--- input :                                              ---
//--- RX 0, 1,2,3 = 32 bit unsigned value                  ---
//--- divisor: our 8bit unsigned value of divide           ---
//------------------------------------------------------------
//--- output:                                              ---
//--- mod10 is touched, rest of divide                     ---
//--- var 1,2,3,4 = RESULT                                 ---
//--- cntr_1 is touched = 0                                ---
//------------------------------------------------------------
div32u_by_8u:
				clr     mod10          // initialize mod10 = 0

				ldi     cntr_1, 32     // lc = 32 (times bits to shift)
divloop:                              // simple 32bit_u divide by const 8bit_u
				lsl     RX0         
				rol     RX1
				rol     RX2
				rol     RX3
				rol     mod10
				cp      mod10, divisor     // divisor value
				brcs    divlp2
				inc     RX0
				sub     mod10, divisor     // divisor value
divlp2:
				dec     cntr_1
				brne    divloop
				ret
//---
//-----------------------------------------------------------------------
//--- copy from STATIC RAM our 32bit value into registers var 1,2,3,4 ---
//-----------------------------------------------------------------------
//--- input : don't care                                              ---
//--- output :                                                        ---
//--- var 1,2,3,4 = 32bit value                                       ---
//--- XL, XH, ZL, ZH, cntr_1 is touched                               ---
//--- cntr_1 = 0                                                      ---
//--- XL,XH  = @(var4 + 1)                                            ---
//--- ZL,ZH  = @(MY_LONG_INT + 4)                                     ---
//-----------------------------------------------------------------------
copy_long_to_registers:
				rcall   set_addr_var
COPY_1:
				ld      R0, Z+
				st      x+, R0
				dec     cntr_1
				brne    COPY_1
				ret
//---
set_addr_var:
				ldi     xl, 1
				clr     xh

set_my_long_addr:

				ldi     zl, low(MY_LONG_INT)
				ldi     zh, high(MY_LONG_INT)
				ldi     cntr_1,4
				ret
//-------------
COPY_BIGC_TO_LONG:
				ldi     XL, low(BIG_CAP)
				ldi     XH, high(BIG_CAP)
				RCALL   set_my_long_addr
				RJMP    COPY_2
//---
COPY_REG_TO_BIGC:
				LDI     XL,1
				CLR     XH
				ldi     zl, low(BIG_CAP)
				ldi     zh, high(BIG_CAP)
				ldi     cntr_1,4
				RJMP    COPY_2
//---
copy_registers_to_long:
				rcall   set_addr_var
COPY_2:
				ld      R0, X+
				st      Z+, R0
				dec     cntr_1
				brne    COPY_2
				ret
//-------------
CLEAR_MY_LONG_INT:
				rcall   set_my_long_addr
				clr     r0
COPY_4:
				st      Z+, r0
				dec     cntr_1
				brne    COPY_4
				ret
//-------------
COPY_REG_TO_FREQ2:
				ldi     zl, low(FREQ2)
				ldi     zh, high(FREQ2)
REG_TO_FREQ:
				ldi     xl, 1
				clr     xh
				ldi     cntr_1,4
				RJMP    COPY_2
//-----------------
COPY_REG_TO_FREQ1:
				ldi     zl, low(FREQ1)
				ldi     zh, high(FREQ1)
				RCALL   REG_TO_FREQ
				
COPY_REG_TO_FREQ3:
				LDS     ZL, FREQ3+2
				TST     ZL
				BRNE    STORE_FR3_HI
				LDS     ZL, FREQ3+1
STORE_FR3_HI:   STS     FR3_HI, ZL  // IF LX MEASSURE CHECK IS CONNECTED INDUCTANCE

				ldi     zl, low(FREQ3)
				ldi     zh, high(FREQ3)
				RCALL   REG_TO_FREQ
				RCALL   FREQ1X_TO_RY

				cp	  RY0, RX0
				cpc	  RY1, RX1
				cpc	  RY2, RX2
				CPC   RY3, RX3

////////////////////////////////////////////////////
//
//				RY = FREQ1 | RX = FREQ 3 
//				IF RY0-3 < RX0-3 C=1
//				IF RY0-3 = RX0-3 C=0 | Z=1
//				IF RY0-3 > RX0-3 C=0 | Z=0
//
/////////////////////////////////////////////////////

				CLR    R0
				CLR    RX7
				ADC    R0, RX7
				STS    SIGN_F, R0
				BRNE   PC+2
				RET
				RCALL  FREQ3_TO_FREQ1X
				RCALL  FREQ1_TO_FREQ3
//--
FREQ1X_TO_LONG:
				ldi     XL, low(FREQ1X)
				ldi     XH, high(FREQ1X)
				RJMP    STORE_TO_LONG
FREQ3_TO_LONG:
				ldi     XL, low(FREQ3)
				ldi     XH, high(FREQ3)
STORE_TO_LONG:
				RCALL   set_my_long_addr
				RJMP    COPY_2
//-----------------------------
//-------------------------
//---  init LCD display ---
//-------------------------




//---------------------
// Lcd = PORTC.3,2,1,0
// RS=PORTC.5
// EN=PORTC.4
// LCD_TYPE = 16 * 2
//---------------------

//--------

//---------------------------------------------------------

//---------------------------------------------------------

        

//---------------------------------------------------------
; PODPROGRAMY OBSUGI WYWIETLACZA LCD W TRYBIE 4-BITOWYM
; Rafa Baranowski (C) 2004
;
; Etykiety procedur:
; 	Ini_LCD		- inicjalizacja wywietlacza
;	Instrukcja_LCD	- przesyanie instrukcji do wywietlacza
;	Zapisz_LCD		- przesyanie danych do wywietlacza
;	Zapisz_CGRAM	- kopiowanie tablicy do pamici CGRAM
;	Czysc_LCD		- czyszczenie wywietlacza
;	Wroc_LCD		- powrt kursora do pozycji pocztkowej
;	Spacja_LCD		- generacja odstpu
;	Pokaz_bajt_LCD	- wywietlanie wartoci bajtu
;	Pokaz_ciag_LCD	- wywietlanie staego acucha znakw
;
; Wymagane stae:
;	K_LCD		- rejestr kierunku portu wyswietlacza
;	O_LCD		- rejestr wyjciowy portu wyswietlacza
;	LCD_LCD_RS	- nr linii dla sygnau LCD_RS
;	LCD_LCD_EN	- nr linii dla sygnau LCD_EN
;	LCD_LCD_DB4	- nr linii dla sygnau LCD_DB4
;	LCD_LCD_DB5	- nr linii dla sygnau LCD_DB5
;	LCD_LCD_DB6	- nr linii dla sygnau LCD_DB6
;	LCD_LCD_DB7	- nr linii dla sygnau LCD_DB7
;
; Wymagane podprogramy dodatkowe:
;	Czekaj_us	- z pliku czekaj_us.inc
;	Czekaj_ms	- z pliku czekaj_ms.inc


;****************************************************************
;		PROCEDURA INICJALIZUJCA WYWIETLACZ LCD
;
LCDIni:
				push	R16
				push	R17

				sbi	K_LCD, LCD_RS	; porty wywietlacza pracuj jako wyjcia
				sbi	K_LCD, LCD_EN
				sbi	K_LCD, LCD_DB4
				sbi	K_LCD, LCD_DB5
				sbi	K_LCD, LCD_DB6
				sbi	K_LCD, LCD_DB7
	
				cbi	O_LCD, LCD_RS	; przesyane wartoci bd instrukcjami
				cbi	O_LCD, LCD_EN

				ldi	R16, 45
				ldi	R17, 1
				rcall	Czekaj_ms		; oczekiwanie przez 45 ms na wypadek, gdyby
								;   zasilanie dopiero zostao wczone

				sbi	O_LCD, LCD_EN
				sbi	O_LCD, LCD_DB4
				sbi	O_LCD, LCD_DB5
				cbi	O_LCD, LCD_DB6
				cbi	O_LCD, LCD_DB7
				nop
				nop
				cbi	O_LCD, LCD_EN	; ustawienie DB7..4=0011 i potwierdzenie
								;   zboczem opadajcym na linii EN
				ldi	R16, 5
				ldi	R17, 1
				rcall	Czekaj_ms		; oczekiwanie przez 5 ms

				ldi	R17, 2
Ini_LCD_1:
				sbi	O_LCD, LCD_EN
				ldi	R16, 1
				rcall	Czekaj_us
				cbi	O_LCD, LCD_EN	
				ldi	R16, 20
				rcall	Czekaj_us
				dec	R17
				brne	Ini_LCD_1		; dwukrotne powtrzenie transmisji tej samej
								;   wartoci (DB7..4=0011) z opnieniem 200 us
								;   (programowa inicjalizacja wywietlacza)

				sbi	O_LCD, LCD_EN
				cbi	O_LCD, LCD_DB4
				ldi	R16, 1
				rcall	Czekaj_us
				cbi	O_LCD, LCD_EN	
				ldi	R16, 10
				rcall	Czekaj_us		; transmisja wartoci DB7..4=0010
								;   (przejcie w tryb interfejsu 4-bitowego)

				ldi	R16, (1<<LCD_USTAW)|(1<<LCD_U_4BIT)|(1<<LCD_U_2LIN)|(1<<LCD_U_5x8)
				rcall	Instrukcja_LCD	; interfejs 4-bitowy, 2 linie, 5x8 pkt

				ldi	R16, (1<<LCD_TRYB)|(1<<LCD_T_INC)
				rcall	Instrukcja_LCD	; tryb zwikszania adresu przy zapisie

				ldi	R16, (1<<LCD_STER)|(1<<LCD_SW_ON)|(1<<LCD_SK_OFF)|(1<<LCD_SM_OFF)
				rcall	Instrukcja_LCD	; wywietlanie wczone, kursor aktywny i miga


				sbi	O_LCD, LCD_RS	; przesyane wartoci bd danymi

				rcall	Czysc_LCD

				pop	R17
				pop	R16
				ret


;****************************************************************
;	PROCEDURA PRZESYAJCA INSTRUKCJ DO WYWIETLACZA
;
; Parametry:
;	R16	- kod instrukcji
;
; Przykad:
;				; wczanie kursora
;	ldi	R16, (1<<LCD_STER)|(1<<LCD_SW_ON)|(1<<LCD_SK_ON)|(1<<LCD_SM_OFF)
;	rcall	Instrukcja_LCD
;

				; etykiety symboliczne bitw tworzcych
				;   kody instrukcji
.EQU	LCD_CZYSC	= 0	; czyszczenie wywietlacza i powrt kursora
.EQU	LCD_WROC	= 1	; powrt kursora do pozycji pocztkowej

.EQU	LCD_TRYB	= 2	; zmiana trybu zapisu pamici
.EQU	LCD_T_INC	= 1	;   adres inkrementowany przy kadym zapisie
.EQU	LCD_T_DEC	= 2	;   adres dekrementowany przy kadym zapisie
.EQU	LCD_T_PRZES	= 0	;   okno przesuwane przy kadym zapisie

.EQU	LCD_STER	= 3	; sterowanie wywietlaczem
.EQU	LCD_SW_ON	= 2	;   wywietlanie wczone
.EQU	LCD_SW_OFF	= 3	;   wywietlanie wyczone
.EQU	LCD_SK_ON	= 1	;   kursor wczony
.EQU	LCD_SK_OFF	= 3	;   kursor wyczony
.EQU	LCD_SM_ON	= 0	;   miganie kursora wczone
.EQU	LCD_SM_OFF	= 3	;   miganie kursora wyczone

.EQU	LCD_PRZES	= 4	; przesuwanie
.EQU	LCD_P_KUR	= 4	;   przesuwanie kursora
.EQU	LCD_P_WYS	= 3	;   przesuwanie okna
.EQU	LCD_LEWO	= 4	;   przesuwanie w lewo
.EQU	LCD_PRAWO	= 2	;   przesuwanie w prawo

.EQU	LCD_USTAW	= 5	; ustawienia interfejsu i wywietlacza
.EQU	LCD_U_8BIT	= 4	;   interfejs 8-bitowy
.EQU	LCD_U_4BIT	= 5	;   interfejs 4-bitowy
.EQU	LCD_U_2LIN	= 3	;   wywietlacz 2-liniowy
.EQU	LCD_U_1LIN	= 5	;   wywietlacz 1-liniowy
.EQU	LCD_U_5x10	= 2	;   znaki o rozmiarach 5x10 punktw
.EQU	LCD_U_5x8	= 5	;   znaki o rozmiarach 5x8 punktw

.EQU	LCD_CGADR	= 6	; zmiana adresu CGRAM
.EQU	LCD_DDADR	= 7	; zmiana adresu DDRAM

Instrukcja_LCD:
				cbi	O_LCD, LCD_RS	; przesyane wartoci bd instrukcjami
				rcall	Zapisz_LCD		; przelij instrukcj zawart w R16
IN_END:
				sbi	O_LCD, LCD_RS	; przesyane wartoci bd danymi

				ret
//---
_from_line2:
				LDI     r24, 0xC0
				RJMP    MAKE_SET
_from_line1:
				LDI     r24, 0x80
//---
MAKE_SET:
				cbi     O_LCD, LCD_RS
				rcall   _Lch
				rjmp    IN_END
;***************************************************************
;			PROCEDURA PRZESYLAJCA BAJT
;
; Parametry:
;	R16	- dane do wysania (np. kod wywietlanego znaku)
_Lch:
				push R16
				mov  r16,r24
				rcall Zapisz_LCD
				pop  r16
				ret
;
Zapisz_LCD:
				sbi	O_LCD, LCD_EN	; linia EN w stanie wysokim
				cbi	O_LCD, LCD_DB4
				cbi	O_LCD, LCD_DB5
				cbi	O_LCD, LCD_DB6
				cbi	O_LCD, LCD_DB7	; linie DB7..4 w stanie niskim
				sbrc	R16, 7
				sbi	O_LCD, LCD_DB7 
				sbrc	R16, 6
				sbi	O_LCD, LCD_DB6 
				sbrc	R16, 5
				sbi	O_LCD, LCD_DB5
				sbrc	R16, 4
				sbi	O_LCD, LCD_DB4	; ustawianie tylko tych linii DB7..4,
				nop				;   ktrym odpowiadaj wysokie wartoci
							;   bitw 7..4 z rejestru R16
				cbi	O_LCD, LCD_EN	; linia EN w stanie niskim - potwierdzenie
							;   (przesanie starszego pbajtu)

				nop
				nop				; opnienie o dwa takty zegarowe

				sbi	O_LCD, LCD_EN
				cbi	O_LCD, LCD_DB4
				cbi	O_LCD, LCD_DB5
				cbi	O_LCD, LCD_DB6
				cbi	O_LCD, LCD_DB7
				sbrc	R16, 3
				sbi	O_LCD, LCD_DB7 
				sbrc	R16, 2
				sbi	O_LCD, LCD_DB6
				sbrc	R16, 1
				sbi	O_LCD, LCD_DB5
				sbrc	R16, 0
				sbi	O_LCD, LCD_DB4	; ustawianie tylko tych linii DB7..4,
				nop				;   ktrym odpowiadaj wysokie wartoci
							;   bitw 3..0 z rejestru R16
				cbi	O_LCD, LCD_EN	; linia EN w stanie niskim - potwierdzenie
							;   (przesanie modszego pbajtu)

				ldi	R16, 5
				rcall	Czekaj_us		; opnienie o 50 us

				ret
;***************************************************************
;		PROCEDURA CZYSZCZCA WYWIETLACZ LCD
;
Czysc_LCD:
				push	R16
				ldi	R16, (1<<LCD_CZYSC)
WYKONAJ_INSTR:
				rcall	Instrukcja_LCD	; czyszczenie wywietlacza i
								;   powrt kursora do pozycji
								;   pocztkowej
				ldi     R16, 180
				rcall   Czekaj_us		; opnienie o 1,8 ms

				pop	R16
				ret


;***************************************************************
;			PROCEDURA POWROTU KURSORA
;
Wroc_LCD:
				push	R16

				ldi	R16, (1<<LCD_WROC)
				RJMP WYKONAJ_INSTR


;***************************************************************
;			PROCEDURA WPROWADZAJCA ODSTP
;
; Parametry:
;	R16	- liczba spacji do wywietlenia
;
ONE_SPACE:
				LDI R16, 1
Spacja_LCD:
				push	R17

				mov	R17, R16	; kopiuj liczb wymaganych spacji do R17
								;   (inicjalizacja licznika ptli)
Spacja_LCD_1:
				ldi	R16, ' '	; R16 zawiera kod ASCII pustego pola (spacji)
				rcall	Zapisz_LCD	; wywietlenie pustego pola (znaku z R16)
				dec	R17		; zmniejszenie licznika ptli
				brne	Spacja_LCD_1; iteracyjne zaptlenie

				pop	R17
				ret



;***************************************************************
;	PROCEDURA WYWIETLAJCA STAY ACUCH ZNAKW
;
; Parametry:
;	Z	- adres pierwszego znaku acucha w pamici programu

.EQU ZNAK_KONCA	= 255 ; etykieta symboliczna znaku koca acucha

WriteLCDString:
				push 	R0
				push	R16
				push	R17

Pok_c_LCD_1:		; ptla wywietlajca kolejne znaki acucha
				lpm	
				adiw	zl,1
				mov		R16,R0
					; odczytaj znak z pamici programu, spod adresu
							;   wskazywanego przez rejestr indeksowy Z
				tst	R16
				breq	Pok_c_LCD_2	; jeli znak jest znakiem koca, to zakocz,
				rcall	Zapisz_LCD	;   w przeciwnym razie wywietl go na LCD
				rjmp	Pok_c_LCD_1	;   i przejd do odczytywania kolejnego znaku

Pok_c_LCD_2:
				pop	R17
				pop	R16
				pop	R0
				ret
//=============================================
;************************************************************
;		MIKROSEKUNDOWA PROCEDURA OPӏNIAJCA
;
; 						Rafa Baranowski (C) 2004
;
; Parametry:
;	R16 - warto opnienia w dziesitkach us (dla zera - 2560 us)
;
; Wymagane stae:
;	SYS_FREQ - czstotliwo pracy w MHz
;
Czekaj_us:
				push	R17
				push	R18

				mov	R18, R16
				ldi	R17, SYS_FREQ

Czekaj_us_0:
				mov	R16, R18		;______
									;      \
Czekaj_us_1:						;       |
				nop					;__     |
				nop					;  \    |
				nop					;   |   |
				nop					;   |   |
				nop					;   p   p
				nop					;   1   0
				nop					;   |   |
				dec	R16				;   |   |
		brne	Czekaj_us_1			;__/    |
									;       |
				dec	R17				;       |
		brne	Czekaj_us_0			;______/

				pop	R18
				pop	R17
				ret
//-------------------
;************************************************************
;		MILISEKUNDOWA PROCEDURA OPӏNIAJCA
;
; 						Rafa Baranowski (C) 2004
;
; Parametry:
;	R16 - warto opnienia w ms (dla zera - 256 ms)
;	R17 - mnonik opnienia (dla zera - 256x)
;
; Wymagane stae:
;	SYS_FREQ - czstotliwo pracy w MHz
;
Czekaj_ms:
				push	R18
				push	R19
				push	R20
				push	R21

				mov	R20, R16
				mov	R21, R17
				ldi	R18, SYS_FREQ

Czekaj_ms_0:
				mov	R17, R21		;______________
								;              \
Czekaj_ms_1:			;              				 |
				mov	R16, R20		;__________     |
								;          \    |
Czekaj_ms_2:					;           |   |
				ldi	R19, 249		;______     |   |
								;      \    |   |
				nop				;       |   |   |
Czekaj_ms_3:					;__     |   |   |
				nop				;  \    p   p   p
				dec	R19			;  p3   2   1   0
				brne	Czekaj_ms_3		;__/    |   |   |
								;       |   |   |
				dec	R16			;       |   |   |
				brne	Czekaj_ms_2		;______/    |   |
								;           |   |
				dec	R17			;           |   |
				brne	Czekaj_ms_1		;__________/    |
								;               |
				dec	R18			;               |
				brne	Czekaj_ms_0		;______________/

				pop	R21
				pop	R20
				pop	R19
				pop	R18
				ret

//==============================================================================
//------------------------------------------------------
RUN_CALIBRATE:
				RCALL    CLEAR_JOB
				RCALL    stop_meassure
				SBI      PORTD, 3 // DISABLE CCALL AND CHARGE BY 2K OHM
				SBI      PORTD, 4 // ENABLE Cx MEAS
//===
//========= MEAS F1
//===
				rcall    LcdIni
				ldi      zl, low(F1_MEAS * 2)
				ldi      zh, high(F1_MEAS * 2) 
				rcall    WriteLCDString 
				RCALL    DO_MEASS
				RCALL    COPY_REG_TO_FREQ1
//===
//============ MEAS F2
//===
				CBI      PORTD, 3 // ENABLE CCALL

				rcall    LcdIni
				ldi      zl, low(F2_MEAS * 2)
				ldi      zh, high(F2_MEAS * 2) 
				rcall    WriteLCDString 
				RCALL    DO_MEASS
				RCALL    COPY_REG_TO_FREQ2
				SBI      PORTD, 3 // DISABLE CCALL
/////////////////////////////////////////////////
				RCALL    CAPP_CALIBRATE
				RCALL    COIL_CALIBRATE
				RET
//-----------------
DO_MEASS:
				rcall   start_meassure 
				TST     R11
				BREQ    PC-1
				RET
//-----------------

//------------------------------------------------------

///////////////////////////////////////////////////////
//                                                   //
//   CAPPACITOR CALCULATE AFTER ADD C_REF 1000 pF    //
//                                                   //
//         C = C_REF * F2^2 / (F1^2 - F2^2)          //
//                                                   //
///////////////////////////////////////////////////////
CAPP_CALIBRATE:
				RCALL FREQ1_TO_FREQ1X
				RCALL FREQ1X_TO_RY     // CALCULATE C IN CIRCUIT BY ADD C_REF 
				RCALL SQUARE_CALC      // RX = F1^2
				RCALL SEND_RX_TO_RM    // STORE F1^2 TO REMAINDER RM = RX => RM = F1^2
				RCALL FREQ2_TO_RY      // RY = F2
				RCALL SQUARE_CALC      // RX = F2^2
				RCALL SUBSTRACT_RM_RX  // RM = RM - RX => RM = F1^2 - F2^2 

				RCALL TRANSFER_RY32_TO_RX // RX = RY = F2
				RCALL CLEAR_RY
				LDS   RY0, C_REF       // 0x3E8 = 1000 pF - CALIBRATION CAPPACITOR VALUE
				LDS   RY1, C_REF+1     // RY = C_REF VALUE | RX = FREQ2
				RCALL MULU32           // RX = FREQ2 * 1000 = (F2*C_REF)
				RCALL FREQ2_TO_RY      // RY = FREQ2
				RCALL MULU32           // RX = FREQ2 * 1000 * FREQ2 = (F2*C_REF*F2) = F2^2 * C_REF
				RCALL SEND_RM_TO_RY    // RM ALL THE TIME = F1^2 - F2^2
				RCALL DIV64            // RX = F2^2 * C_RE / (F1^2 - F2^2) => RX = VALUE OF pF CAPP IN THE CIRCUIT
				STS   C_OWN, RX0
				STS   C_OWN+1, RX1
				RET

///////////////////////////
// END OF CAPP CALIBRATE //
///////////////////////////



///////////////////////////////////////////////////////
//                                                   //
//      CALCULATE INDUCTANCE OF CIRCUIT COIL         //
//                                                   //
//              L = 1/ 4*Pi^2*C*F1^2                 //
//                                                   //
//              L = 1/4/Pi/Pi/C/F1/F1                //
//                                                   // 
///////////////////////////////////////////////////////
//              THE TRICK OF MAGIC_NUMBER            //
//                                                   //
//        MAGIC_NUMBER = 2533029591058444286 =       //
//                                                   //
//      100000000000000000000/4/Pi/Pi          //
//                                                   //
//              = 0x232720FE14976FFE                 //
//                                                   //
///////////////////////////////////////////////////////
COIL_CALIBRATE:
				RCALL FREQ1_TO_FREQ1X
				RCALL SET_RX_MAGIC_NUMBER  // RX = 0x232720FE14976FFE 
				RCALL FREQ1X_TO_RY         // RY = F1
				RCALL DIV64                // RX = RX/F1
				RCALL DIV64                // RX = RX/F1/F1
				RCALL CLEAR_RY             // RY = 0

				LDS   RY0, C_OWN           // 0x3D2 = 978 pF
				LDS   RY1, C_OWN+1
				RCALL DIV64                // RX = RX/F1/F1/C
				STS   L_OWN, RX0
				STS   L_OWN+1, RX1
				RET
///////////////////////////
// END OF COIL CALIBRATE //
///////////////////////////

///////////////////////////////
// COIL CALCULATE INDUCTANCE //
///////////////////////////////
LX_CALC:
//---------------------------
				RCALL FIRST_CALC

				LDS   RY0, L_OWN      // 0x279B = 101,13 uH - CALIBRATION CAPPACITOR VALUE
				LDS   RY1, L_OWN+1    // RY = 10113 RX = (F1^2 - F3^2)/F3
SECOND_CALC:
				RCALL MULU32          // RX = [(F1^2 - F3^2)/F3] * 10113 |  [L=101,13 uH]
				RCALL FREQ3_TO_RY     // RY = FREQ3
				RCALL DIV64           // RX = [(F1^2 - F3^2)/F3] * 10113 / F3 => RX = VALUE OF uH ADDED TO THE CIRCUIT
				RJMP  copy_registers_to_long
FIRST_CALC:
				RCALL FREQ1X_TO_RY      // CALCULATE C IN CIRCUIT BY ADD C_REF 
				RCALL SQUARE_CALC      // RX = F1^2
				RCALL SEND_RX_TO_RM    // STORE F1^2 TO REMAINDER RM = RX => RM = F1^2
				RCALL FREQ3_TO_RY      // RY = FREQ3 |[L]
				RCALL SQUARE_CALC      // RX = F2^2
				RCALL SUBSTRACT_RM_RX  // RM = RM - RX => RM = F1^2 - F3^2 |[L}
				RCALL TRANSFER_RM_TO_RX // RX = RM =  F1^2 - F3^2 |[L]
				RCALL DIV64            // RX = (F1^2 - F3^2)/F3
				RJMP  CLEAR_RY
//====================================================
CX_CALC:
				RCALL FIRST_CALC             // RX = (F1^2 - F3^2)/F3

                LDI   RM5, 10
				MOV   RY0, RM5
				RCALL MULU32 

				LDS   RY0, C_OWN      // RY = 978 pF - CALIBRATION CAPPACITOR VALUE
				LDS   RY1, C_OWN+1
				RCALL MULU32 
				RCALL FREQ3_TO_RY
				RCALL DIV64 
				RJMP  copy_registers_to_long

///////////////////////////////////////////
//             SQUARE 32BIT              //
///////////////////////////////////////////
SQUARE_CALC:
				RCALL TRANSFER_RY32_TO_RX
MULU32:
//////////////////////////////////////////////////////////
//           MULT32
//
// INPUT:
// RX0 to RX3
// RY0 TO RY3
//
// OPERATION:
// RX32 * RY32 [RX0 to RX3] * [RY0 to RY3]
//
// USED:
// R0 AS COUNTER
// RX64BIT RX0 to RX7
// RY0 TO RY3
//
// RESULT:
// RX = RX * RY
// ******************
// * RX = RX mul RY *
// ******************
// **********************************
// * RX64BIT RX0 to RX7 = RX mul RY *
// **********************************
// RY0 to RY3 DON'T TOUCHED
// R0 = 0
//////////////////////////////////////////////////////////

				RCALL CLEAR_HIGH_RX
				PUSH  R25
				LDI   R25,33
				MOV   R0, R25
				POP   R25
				SUB   RX4,RX4
				brcc  PC+5
				add   RX4,RY0
				adc   RX5,RY1
				adc   RX6,RY2
				adc   RX7,RY3
				ror   RX7
				ror   RX6
				ror   RX5
				ror   RX4
				ror   RX3
				ror   RX2
				ror   RX1
				ror   RX0
				dec   R0
				brne  PC-14
				ret
///////////////////////////////////////////////////////////////////////////
//				Division 64 / 48 bit
//
// INPUT:
// RX0 TO RX7
// RY0 TO RY5
//
// USED:
// RM0 TO RM5
// RX0 TO RX7
// RY0 TO RY5
// R0 AS COUNTER
//
// OPERATION:
// RX = RX/RY
// RX 64BIT DIV RY 48BIT
// RX0 to RX7 DIV RY0 to RY5
// RM 48BIT 
//
// RESULT:
// **************
// * RX = RX/RY *
// **************
// * RX0 to RX7 *
// **************
// RY DON'T TOUCHED
// R0 = 0
// RM = RX mod RY
//-------------------------------------------------------------------------
DIV64:	 		

				clr	  RM0
				clr	  RM1
				clr	  RM2
				CLR   RM3
				ldi	  RM4, 64
				MOV   R0, RM4 //COUNTER
				CLR   RM4
				CLR   RM5
//PC-30
				lsl	  RX0
				rol	  RX1
				rol   RX2
				rol	  RX3
				rol	  RX4
				rol	  RX5
				rol	  RX6
				ROL   RX7

				rol	  RM0
				rol	  RM1
				rol	  RM2
				ROL   RM3
				ROL   RM4
				ROL   RM5
				brcs	PC+8

				cp	  RM0, RY0
				cpc	  RM1, RY1
				cpc	  RM2, RY2
				CPC   RM3, RY3
				CPC   RM4, RY4
				CPC   RM5, RY5
				brcs	PC+8
//PC+8
				sub	  RM0, RY0
				sbc	  RM1, RY1
				sbc	  RM2, RY2
				SBC   RM3, RY3
				SBC   RM4, RY4
				SBC   RM5, RY5
				inc	  RX0
//PC+8
				dec	  R0
				brne   PC-30
				ret
;------------------------------------------------------------------------
CLEAR_HIGH_RX:
				CLR RX7
				CLR RX6
				CLR RX5
				CLR RX4
				RET
//-----------------
CLEAR_RY:
				CLR RY0
				CLR RY1
				CLR RY2
				CLR RY3
				CLR RY4
				CLR RY5
				RET
//-----------------
SEND_RX_TO_RM:

				MOV RM0, RX0
				MOV RM1, RX1
				MOV RM2, RX2
				MOV RM3, RX3
				MOV RM4, RX4
				MOV RM5, RX5
				RET
//-----------------
SEND_RM_TO_RY:

				MOV RY0, RM0
				MOV RY1, RM1
				MOV RY2, RM2
				MOV RY3, RM3
				MOV RY4, RM4
				MOV RY5, RM5
				RET
//-----------------
TRANSFER_RM_TO_RX:
				RCALL CLEAR_HIGH_RX
				MOV RX0, RM0
				MOV RX1, RM1
				MOV RX2, RM2
				MOV RX3, RM3
				MOV RX4, RM4
				MOV RX5, RM5
				RET
//-----------------
SUBSTRACT_RM_RX:
				sub	  RM0, RX0
				sbc	  RM1, RX1
				sbc	  RM2, RX2
				SBC   RM3, RX3
				SBC   RM4, RX4
				SBC   RM5, RX5
				RET
//-----------------
TRANSFER_RY32_TO_RX:
				MOV   RX0, RY0
				MOV   RX1, RY1
				MOV   RX2, RY2
				MOV   RX3, RY3
				RET
//---------------------
SET_RY_8M:
				ldi     ZL, low(STATIC8M * 2)
				ldi     ZH, high(STATIC8M * 2) 
				PUSH    R16
				LDI     R16,6
				ldi     XL, LOW(RY_ADD)
				CLR     XH
				RJMP    SAVE_TO_REGS
SET_RX_MAGIC_NUMBER:
				ldi     ZL, low(MAGIC_NO * 2)
				ldi     ZH, high(MAGIC_NO * 2) 
				PUSH    R16
				LDI     R16,8
				ldi     XL, LOW(RX_ADD)
				CLR     XH
SAVE_TO_REGS:
				lpm
				adiw	zl,1
				st      X+, R0
				DEC     R16
				BRNE    PC-4
				POP     R16
				RET
//------------------
COPY_VARIABLES:
				ldi     ZL, low(CREFX * 2)
				ldi     ZH, high(CREFX * 2) 
				PUSH    R16
				LDI     R16, LOW(DAT_END*2 - CREFX*2)
				ldi     xl, LOW(C_REF)
				ldi     XH, HIGH(C_REF)	
				RJMP    SAVE_TO_REGS
//------------------
FREQ1_TO_FREQ3:
				ldi    	ZL, low(FREQ1)
				ldi     ZH, high(FREQ1)
				ldi    	XL, low(FREQ3)
				ldi     XH, high(FREQ3)
				RJMP    STORE4BYTES
FREQ3_TO_FREQ1X:
				ldi    	ZL, low(FREQ3)
				ldi     ZH, high(FREQ3)
				RJMP    STORE_FREQ1X
FREQ1_TO_FREQ1X:
				ldi    	ZL, low(FREQ1)
				ldi     ZH, high(FREQ1)
STORE_FREQ1X:
				ldi    	XL, low(FREQ1X)
				ldi     XH, high(FREQ1X)
				RJMP    STORE4BYTES
FREQ1X_TO_RY:
				ldi    	zl, low(FREQ1X)
				ldi     zh, high(FREQ1X)
STORE_FREQ: 
				RCALL   CLEAR_RY
				ldi     XL, LOW(RY_ADD)
				CLR     XH
STORE4BYTES:
				PUSH    R16
				LDI     R16,4

				LD      R0, Z+
				ST      X+, R0
				DEC     R16
				BRNE    PC-3
				POP     R16
				RET
//------------------
FREQ2_TO_RY:
				ldi    	zl, low(FREQ2)
				ldi     zh, high(FREQ2) 
				RJMP    STORE_FREQ
//-------------------------------
FREQ3_TO_RY:
				ldi    	zl, low(FREQ3)
				ldi     zh, high(FREQ3) 
				RJMP    STORE_FREQ
//-----------------------------------

//---
GET_KEY_STATE3:
				PUSH    R16
				PUSH    R28
				LDI     R28, 31

GET_KEY_STATE2:
				DEC     R28
				BRNE    CHK_STILL
				rjmp    _RESET_PROC
CHK_STILL:
				RCALL   WAIT50MS
				IN      R16, PIND
				ANDI    R16, 0x80
				CPI     R16, 0x80
				BRNE    GET_KEY_STATE2
				RCALL   WAIT50MS
				POP     R28
				POP     R16
				RET
//---
WAIT250MS:
				LDI     R16, 250
				RJMP    STILL_WAIT
//---
WAIT50MS:
				ldi	    R16, 50
STILL_WAIT:
				ldi	    R17, 1
				RJMP	Czekaj_ms
//---

//------------------------------------
CHECK_FREQ:
				LDS     R0, FR3_HI
				TST     R0
				BRNE    CHECK_FREQ_EXIT
				rcall    LcdIni
				RCALL    _from_line2
				ldi      zl, low(NO_LX * 2)
				ldi      zh, high(NO_LX * 2) 
				rcall    WriteLCDString	
				CLR      R0			
CHECK_FREQ_EXIT:
				RET
//------------------------------------

KEYBOARD_SERVICE:

				RCALL    GET_KEY_STATE3   // WAIT FOR UNPRESS KEY
INCREMENT_JOB:
				LDS      R16, MY_JOB
				INC      R16
				ANDI     R16, 3
				RJMP     STORE_JOB
CLEAR_JOB:
				CLR      R16
STORE_JOB:
				STS      MY_JOB, R16    // CHANGE MY_JOB
				RET
//------------------------------------


//=================================
				
//====================================
CE_MEASSURE:
				CBI      PORTD, 4 // STOP FREQ

                LDS      R16, MY_JOB
				CPI      R16, 3
                BREQ     BIG_CX_MEASS

				SBI      PORTD, 3 // STOP BIG CHARGE K2 IN START POSSITION
				RCALL    DISCHARGE

				RCALL    SMALL_CHARGE
WAIT_SMALL:
				MOV      R24, R3
				CPI      R24, 0x80 //64   // 1 SEC NOT CHARGE ?
				BRCS     WSM2
			    // TO BIG CAPP. GO TO BIG CHARGE
				RCALL    INCREMENT_JOB
				RJMP     BIG_CX_MEASS
WSM2:
//--------------------
//- KEYBOARD SERVICE -
//--------------------
				SBIS     PIND,7     // SW1 PRESSED?
				RJMP     CHANGE_JOB
//--------------------
				TST      R10
				BREQ     WAIT_SMALL
				SBI      PORTD, 6 // START DISCHARGE IMMEDIATELLY
				RCALL    SMALL_CX_PRINT
				RJMP     starter

BIG_CX_MEASS:
				CBI      PORTD, 4 // STOP FREQ
				RCALL    DISCHARGE
				RCALL    DISCHARGE

                LDS      R16, MY_JOB
				CPI      R16, 2
                BREQ     CE_MEASSURE

				RCALL    BIG_CHARGE
WAIT_BIG:
//--------------------
//- KEYBOARD SERVICE -
//--------------------
				SBIS     PIND,7     // SW1 PRESSED?
				RJMP     CHANGE_JOB
//--------------------
				TST      R10
				BREQ     WAIT_BIG
				SBI      PORTD, 6 // START DISCHARGE IMMEDIATELLY
				RCALL    COPY_REG_TO_BIGC
				RCALL    BIG_CX_PRINT
				RJMP     starter
//=================================================
CHANGE_JOB:
				RCALL    KEYBOARD_SERVICE
				ldi      yh,high(RAMEND)
				out      SPH,yh  // stack init
				ldi      yl,low(RAMEND)
				out      SPL,yl  // stack init
				dec      yh
				RCALL    STOP_DISCHARGE // STOP DISCHARGE
				RJMP     starter
//==================================================
//---

//---
BIG_CX_PRINT:
				RCALL    CLEAR_RY
				LDI      R24, 125
				MOV      RY0, R24
				RCALL    MULU32
				LDS      RY0, REZ3
				LDS      RY1, REZ3+1
				RCALL    DIV64
				CLR      RY1
				LDI      R24, 10
				MOV      RY0, R24
				RCALL    DIV64

				RCALL    copy_registers_to_long
				RCALL    CE_M_PR
				RCALL    get_string_pointers

				CPI      R17,4
				BRCS     MEDIUM_UF
				SUBI     R17, 3
				CLR      R18
				RCALL    print_from_buff
				RCALL    print_dot
				LDI      R17,3
				CLR      R18
				RCALL    print_from_buff
				RCALL    CE_MF_PR
				RET
//---
MEDIUM_UF:
				PUSH     R17
				RCALL    print_meassure_0digits
				RCALL    CE_UF_PR
				POP      R17
				CPI      R17, 3
				BRCS     SWITCH_RANGE // SMALL CAPP AUTOSWTCHING RANGE
				RET
SWITCH_RANGE:
                CPI      R17, 2
				BREQ     CHG_JOB
				LDS      R17, MY_LONG_INT
				CPI      R17, 0
				BREQ     NO_CHG
CHG_JOB:
				LDI      R17,2
				STS      MY_JOB, R17
				LDI      R28, 8
S2_LP:
				RCALL    WAIT250MS
				DEC      R28
				BRNE     S2_LP
NO_CHG:			RET
//------------------------------

//==============================================
SMALL_CX_PRINT:
				RCALL    CLEAR_RY
				LDI      R24, 125
				MOV      RY0, R24
				RCALL    MULU32
				LDS      RY0, REZ1
				LDS      RY1, REZ1+1
				RCALL    DIV64
				CLR      RY1
				LDI      R24, 10
				MOV      RY0, R24
				RCALL    DIV64

				RCALL    copy_registers_to_long
				RCALL    CE_M_PR
				RCALL    get_string_pointers

				CPI      R17, 1
				BRNE     TEST_2_DG
				RCALL    print_0_dot
				RCALL    print_0
				RCALL    print_last_digit
				RJMP     CE_UF_PR
//---
TEST_2_DG:		CPI      R17, 2
				BRNE     TEST_3_DG
				RCALL    print_0_dot
				RCALL    print_two_last_digit
				RJMP     CE_UF_PR
TEST_3_DG:
				RCALL    print_meassure_2digits
				RJMP     CE_UF_PR
//================================================
CE_M_PR:
				rcall    LcdIni
				RCALL    _from_line2
				LDS      R16, MY_JOB
				CPI      R16,2
				BRNE     BIG_INF
				ldi      ZL, low(LOW_CE * 2)
				ldi      ZH, high(LOW_CE * 2) 
				RCALL    WriteLCDString
				RJMP     CE_MPR
BIG_INF:
				ldi      ZL, low(HI_CE * 2)
				ldi      ZH, high(HI_CE * 2) 
				RCALL    WriteLCDString
CE_MPR:
				RCALL    _from_line1
				ldi      ZL, low(CE_M * 2)
				ldi      ZH, high(CE_M * 2)
				RJMP     WriteLCDString
//---
CE_UF_PR:
				ldi      ZL, low(CE_UF * 2)
				ldi      ZH, high(CE_UF * 2)
				RJMP     WriteLCDString
CE_MF_PR:
				ldi      ZL, low(CE_MF * 2)
				ldi      ZH, high(CE_MF * 2)
				RJMP     WriteLCDString
//---
//----------------------------------
// R3 =100 * R2 = 20000 -
//----------------------------------
R3_CALC:
				RCALL    CLEAR_RY

				LDS      RY0, REZ2
				LDS      RY1, REZ2+1
				RCALL    TRANSFER_RY32_TO_RX
				RCALL    CLEAR_RY
				PUSH     R16
				LDI      R16, 100
				MOV      RY0, R16
				POP      R16
				RCALL    MULU32
				STS      REZ3, RX0
				STS      REZ3+1, RX1
				RET
//------------------------
//--- block interrupts ---
//------------------------
CE_START:
				rcall    stop_meassure
				rcall    CLEAR_MY_LONG_INT
				RCALL    copy_long_to_registers
				CLI
//---------------------------------
//--------------------------------------------------------

//--------------------------------------------------------

				LDI      ZL, (1<<ISC01) | (0<<ISC00)
#ifdef m328p
				sts      EICRA, ZL                // ENABLE INT0 FALLING EDGE
				LDI      ZL, (1<<INTF0)
				out      EIFR, ZL                 // CLR IRQ FLAG
				ldi      ZL, TIMSK1_START_VALUE
				sts      TIMSK1, ZL
				ldi      ZL, (1<<INT0)
				out      EIMSK, ZL
#else
				out      MCUCR, ZL                  // ENABLE INT0 FALLING EDGE
				LDI      ZL, (1<<INTF0)				
				out      GIFR, ZL                   // CLR IRQ FLAGS
				ldi      ZL, (1<<TOIE1)
				out      TIMSK,  ZL
				ldi      ZL, (1<<INT0)
				OUT      GICR, ZL
#endif
				ret
//--------------
DISCHARGE:
				CBI      DDRD, 2  // INPUT CE_T
				SBI      PORTD, 2 // INPUT CE_T
				SBI      DDRD, 3  // OUTPUT K2
				SBI      DDRD, 4  // OUTPUT K1
				SBI      DDRD, 6  // OUTPUT BIGCHRG
				SBI      PORTD, 6 // START DISCHARGE
DSCHG_LOOP:
//--------------------
//- KEYBOARD SERVICE -
//--------------------
				SBIS     PIND,7     // SW1 PRESSED?
				RJMP     CHANGE_JOB
//--------------------
				
				RCALL    READ_ADC
				TST      R17
				BRNE     DSCHG_LOOP
				CPI      R16, 10
				BRCC     DSCHG_LOOP
				RCALL    WAIT50MS
				SEI
				RET
//=================================
SMALL_CHARGE:
				RCALL    CE_START
				CLR      R10
				RCALL    CLEAR_IRQ_FLAGS
				LDI      ZL,(1<<CS10)
#ifdef m328p
				sts      TCCR1B, ZL                 // START TIMER1
#else
				out      TCCR1B, ZL                 // START TIMER1
#endif
				SEI
				SBI      PORTD, 3    // SMALL CHARGE K2 IN START POSSITION
STOP_DISCHARGE:
				CBI      PORTD, 6    // STOP DISCHARGE
				RET
//=================================
BIG_CHARGE:
				RCALL    CE_START
				CLR      R10
				RCALL    CLEAR_IRQ_FLAGS
#ifdef m328p
				ldi      ZL,(1<<CS10)
				sts      TCCR1B, ZL                 // START TIMER1
#else
				ldi      ZL,(1<<CS10)
				out      TCCR1B, ZL                 // START TIMER1
#endif
				CBI      PORTD, 3    // BIG CHARGE
				CBI      PORTD, 6    // STOP DISCHARGE
				SEI
				RET
//-------------------------------------
READ_ADC:

				CLI
				RCALL    stop_meassure

				CBI DDRC, 0
				CBI PORTC, 0
//---
				clr      ZL
#ifdef m328p
				out      SMCR , ZL
				STS      ADMUX, ZL
				STS      ADCSRA, ZL


				LDI      R16, ADMUX_START_VALUE
				STS      ADMUX, R16
				LDI      R16, ADCSR_START_VALUE
				STS      ADCSRA, R16
				LDI      R16, SMCR_START_VALUE
				OUT      SMCR, R16
				SEI

				LDI      R16, 100
				RCALL    STILL_WAIT
				SLEEP

				LDS      R1, ADCL
				PUSH     R1
				LDS      R1, ADCH
				PUSH     R1
				CLI
				RCALL    stop_meassure
				clr      ZL
				out      SMCR , ZL
				STS      ADMUX, ZL
				STS      ADCSRA, ZL
//---
#else
				IN       R16, MCUCR
				ANDI     R16, 0x0F
				OUT      MCUCR, R16 
				OUT      ADMUX, ZL
				OUT      ADCSRA, ZL

 
				IN       R16, MCUCR
				SBR      R16, (1<<SE)|(1<<SM0)
				OUT      MCUCR, R16
				LDI      R16, (1<<ADEN)|(1<<ADIE)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0)
				OUT      ADCSRA, R16
				LDI      R16, (1<<REFS0)
				OUT      ADMUX, R16

				SEI

				LDI      R16, 200
				RCALL    STILL_WAIT
				SLEEP

				IN      R16, ADCL
				PUSH     R16
				IN      R1, ADCH
				PUSH     R17

				CLI
				RCALL    stop_meassure
				clr      ZL
				OUT      ADMUX, ZL
				OUT      ADCSRA, ZL
				IN       R16, MCUCR
				ANDI     R16, 0x0F
				OUT      MCUCR, R16
#endif
//================================================
				SEI
				POP      R17
				POP      R16
				RET
//***********************************************
//--------------------------
//-- a start info string ---
//--------------------------
welcome:
				.DB "HILAND L/C METER" ,0,0
welcome2:
				.DB "  2019-11-25 ",0
welcome3:
				.DB " WEGI - REMARK ",0
//-------------------
F1_MEAS:
				.DB " MEASURE F1... " , 0
F2_MEAS:
				.DB " MEASURE F2... " , 0
CAL_OK:
				.DB " CALIBRATED", 0

F1_EQU:			.DB "F1=",0
F2_EQU:			.DB "F2=",0

NO_LX:			.DB "Lx = 0.00 uH",0,0

CE_M:			.DB "CE = ",0
CE_UF:          .DB " uF",0
CE_MF:          .DB " mF",0
LOW_CE:         .DB "LO [CE<500uF]",0
HI_CE:          .DB "HI [CE>500uF]",0
//==========================================
// !!! FREQUENCY MUST BE CHANGED TO PHYSICAL CRYSTAL !!!
STATIC8M:
    .DB 0x00, 0x12, 0x7A, 0x00, 0x00, 0x00  // CLOCK FREQ 8 000 000 VALUE
    // .DB 0x00, 0x2D, 0x31, 0x01, 0x00, 0x00  // CLOCK FREQ 20 000 000 VALUE
MAGIC_NO:
	.DB 0xFE, 0x6F, 0x97, 0x14, 0xFE, 0x20, 0x27, 0x23
CREFX:
	.DB 0x08, 0x07             // C_REF 1800 pF CALIBRATE VALUE
	.DB 0x1B, 0x07             // C_OWN 1819  pF CIRCUIT   VALUE
	.DB 0x7B, 0x1C             // L_OWN 72.91 uH (7291)
	.DB 0xD0, 0x07             // R1 = 2000 OHM
	.DB 0xC8, 0x00             // R2 = 200  OHM
DAT_END:
//==========================================







//-------
//- EOF -
//-------
